/*
 * SUSBDMXTWODRV.H -- include file for SUSBDMXTWODRV.DLL
 *   to communicate with Soundlight USBDMX-TWO usb to dmx512 interface.
 *
 * Copyright (C) SOUNDLIGHT
 * All rights reserved 
 */

#ifndef SUSBDMXTWODRV_H
#define SUSBDMXTWODRV_H

#if defined __cplusplus
extern "C" {
#endif

// define the dll space this file is used in
#ifdef SUSBDMXTWODRV_DLL_EXPORT
#define SUSBDMXTWODRV_TYPE	__declspec(dllexport) __stdcall
#else
#define SUSBDMXTWODRV_TYPE	__declspec(dllimport) __stdcall
#endif

/*
 * DLL version
 */
static const USHORT SUSBDMXTWODRV_DLL_VERSION = 0x0407;

/*
 * MACRO to verify dll version
 */
#define SUSBDMXTWODRV_DLL_VERSION_CHECK() (susbdmxtwodrv_version() >= SUSBDMXTWODRV_DLL_VERSION)

/* *************************************
 * functions defined in the usbdmx.dll *
 ***************************************/

/*
 *   susbdmxtwodrv_version(): returns the version number (16bit, 4 digits BCD)
 * Current version is SUSBDMXTWODRV_DLL_VERSION. Use the Macro 
 * SUSBDMXTWODRV_DLL_VERSION_CHECK() compare dll's and header files version.
 */
USHORT	SUSBDMXTWODRV_TYPE	susbdmxtwodrv_version();

/*
 *   susbdmxtwodrv_open(): open device number <device>, where 0 is the first
 * and unlimit devices are supported. The function returnes FALSE
 * if <device> is not supported. Use returned handle h to access the 
 * device later on. One device can be opened an unlimited number of times.
 */
BOOL	SUSBDMXTWODRV_TYPE	susbdmxtwodrv_open(USHORT device, PHANDLE h);

/*
 *   susbdmxtwodrv_close(): close the device identified by the given handle.
 */
BOOL	SUSBDMXTWODRV_TYPE	susbdmxtwodrv_close(HANDLE h);

/*
 *   susbdmxtwodrv_device_id(): read the device id of the device
 */
BOOL	SUSBDMXTWODRV_TYPE	susbdmxtwodrv_device_id(HANDLE h, PUSHORT pid);

/*
 *   susbdmxtwodrv_is_XXX(): identify the device identified by the given handle.
 * Each function returns TRUE if the device matches.
 */
BOOL	SUSBDMXTWODRV_TYPE	susbdmxtwodrv_is_usbdmxtwo(HANDLE h);

/*
 *   susbdmxtwodrv_product_get(): read the product string from the device.
 * size specifies the maximum size of the buffer pointed to by <string> 
 * (unit bytes).
 */
BOOL	SUSBDMXTWODRV_TYPE	susbdmxtwodrv_product_get(HANDLE h, PWCHAR string, USHORT size);

/*
 *   susbdmxtwodrv_serial_get(): read the serial number from the device.
 * size specifies the maximum size of the buffer pointed to by <string> 
 * (unit bytes).
 */
BOOL	SUSBDMXTWODRV_TYPE	susbdmxtwodrv_serial_get(HANDLE h, PWCHAR string, USHORT size);

/*
 *   susbdmxtwodrv_device_version(): Read the the device version of a device.
 * the device version is one of the values within the USBs configuration
 * descriptor (BcdDevice). pversion is only valid if the function returns
 * TRUE.
 */
BOOL	SUSBDMXTWODRV_TYPE	susbdmxtwodrv_device_version(HANDLE h, PUSHORT pversion);

/*
 * susbdmxtwodrv_name_[set|get]
 */
BOOL	SUSBDMXTWODRV_TYPE	susbdmxtwodrv_name_set(IN HANDLE h, IN  PCSTR string);
BOOL	SUSBDMXTWODRV_TYPE	susbdmxtwodrv_name_get(IN HANDLE h, OUT  PSTR string, IN USHORT length);

/*
 * susbdmxtwodrv_intensity_[set|get]
 */
BOOL	SUSBDMXTWODRV_TYPE	susbdmxtwodrv_intensity_set(IN HANDLE h, IN   UINT  intensity);
BOOL	SUSBDMXTWODRV_TYPE	susbdmxtwodrv_intensity_get(IN HANDLE h, OUT PUINT pintensity);

/*
 * susbdmxtwodrv_speed_[set|get]
 */
BOOL	SUSBDMXTWODRV_TYPE	susbdmxtwodrv_speed_set(IN HANDLE h, IN   UINT  speed);
BOOL	SUSBDMXTWODRV_TYPE	susbdmxtwodrv_speed_get(IN HANDLE h, OUT PUINT pspeed);

/*
 * susbdmxtwodrv_sync_config_[set|get]
 */
BOOL	SUSBDMXTWODRV_TYPE	susbdmxtwodrv_sync_config_set(IN HANDLE h, IN   UCHAR  SyncMode, IN   UCHAR  SyncIntensity, IN   UCHAR  SyncSpeed);
BOOL	SUSBDMXTWODRV_TYPE	susbdmxtwodrv_sync_config_get(IN HANDLE h, OUT PUCHAR pSyncMode, OUT PUCHAR pSyncIntensity, OUT PUCHAR pSyncSpeed);

/*
 * susbdmxtwodrv_sync_group_[set|get]
 */
BOOL	SUSBDMXTWODRV_TYPE	susbdmxtwodrv_sync_group_set(IN HANDLE h, IN   UCHAR  SyncGroup);
BOOL	SUSBDMXTWODRV_TYPE	susbdmxtwodrv_sync_group_get(IN HANDLE h, OUT PUCHAR pSyncGroup);

/*
 * susbdmxtwodrv_firmware_version - return device structures firmware version incl. build revision
 */
BOOL	SUSBDMXTWODRV_TYPE	susbdmxtwodrv_firmware_version(IN HANDLE h, OUT PUCHAR pmajor, OUT PUCHAR pminor, OUT PUSHORT prevision);

/*
 * susbdmxtwodrv_network_[set|get] - set/get IPv4 network configuration
 */
BOOL	SUSBDMXTWODRV_TYPE	susbdmxtwodrv_network_set(IN HANDLE h, IN   UINT  ip, IN   UINT  mask, IN   UINT  gateway);
BOOL	SUSBDMXTWODRV_TYPE	susbdmxtwodrv_network_get(IN HANDLE h, OUT PUINT pip, OUT PUINT pmask, OUT PUINT pgateway);

/*
 * susbdmxtwodrv_network_v6_[set|get] - set/get IPv6 network configuration
 */
BOOL	SUSBDMXTWODRV_TYPE	susbdmxtwodrv_network_v6_set(IN HANDLE h, IN  PUCHAR pip, IN  PUCHAR pmask, IN  PUCHAR pgateway);
BOOL	SUSBDMXTWODRV_TYPE	susbdmxtwodrv_network_v6_get(IN HANDLE h, OUT PUCHAR pip, OUT PUCHAR pmask, OUT PUCHAR pgateway);

/*
 * susbdmxtwodrv_network_mac_get - read network MAC address
 */
BOOL	SUSBDMXTWODRV_TYPE	susbdmxtwodrv_network_mac_get(IN HANDLE, OUT PUCHAR pMAC);

/*
 * susbdmxtwodrv_time_[set|get] - set time and date in plexus
 */
BOOL	SUSBDMXTWODRV_TYPE	susbdmxtwodrv_time_set(IN HANDLE h, IN  UCHAR   seconds, IN   UCHAR  minutes, IN   UCHAR  hours, 
										   IN  UCHAR   day_of_week, IN   UCHAR  day_of_month, IN   UCHAR  month, IN   USHORT  year);
BOOL	SUSBDMXTWODRV_TYPE	susbdmxtwodrv_time_get(IN HANDLE h, OUT PUCHAR pseconds, OUT PUCHAR pminutes, OUT PUCHAR phours,
										   OUT PUCHAR pday_of_week, OUT PUCHAR pday_of_month, OUT PUCHAR pmonth, OUT PUSHORT pyear);

/*
* susbdmxtwodrv_geo_coordinates_[set|get] - set geo cordinates for calculate the sunset and sunrise time in plexus
*/
BOOL	SUSBDMXTWODRV_TYPE	susbdmxtwodrv_geo_coordinates_set(IN HANDLE, 
															  IN SHORT LatitudeDegree,  IN UCHAR LatitudeMinutes,  IN UCHAR LatitudeSeconds,
															  IN SHORT LongitudeDegree, IN UCHAR LongitudeMinutes, IN UCHAR LongitudeSeconds,
															  IN UCHAR TimeZoneOffsetIndex );
BOOL	SUSBDMXTWODRV_TYPE	susbdmxtwodrv_geo_coordinates_get(IN HANDLE, 
															  OUT PSHORT pLatitudeDegree,  OUT PUCHAR pLatitudeMinutes,  OUT PUCHAR pLatitudeSeconds,
															  OUT PSHORT pLongitudeDegree, OUT PUCHAR pLongitudeMinutes, OUT PUCHAR pLongitudeSeconds,
															  OUT PUCHAR pTimeZoneOffsetIndex);

/*
 * susbdmxtwodrv_card_info - return card information, sizemb is 0 if not card is available
 */
BOOL	SUSBDMXTWODRV_TYPE	susbdmxtwodrv_card_info(IN HANDLE h, OUT PUINT psizemb, OUT PUINT pfreemb, OUT PINT preadstate, OUT PUINT pwpstate);
// readstate can have this values in netagive, 0: no file, 1..99: loading, 100: file loaded
enum {
	CARD_INFO_READSTATE_NO_ERROR = 0,		// no error
	CARD_INFO_READSTATE_STARTTAG,			// unsupported starttag in config file
	CARD_INFO_READSTATE_STARTTAG_PLXCONFIG,	// unsupproted starttag in <plxconfig>
	CARD_INFO_READSTATE_ATTR_PLXCONFIG,		// unsupported attribute in <plxconfig>
	CARD_INFO_READSTATE_ATTR_SCENE,			// unsupported attribute in <scene>
	CARD_INFO_READSTATE_ATTR_TIMER,			// unsupported attribute in <timer>
	CARD_INFO_READSTATE_ATTR_PORT,			// unsupported attribute in <port>
	CARD_INFO_READSTATE_NUM_TIMER,			// to many timer
	CARD_INFO_READSTATE_NUM_PORTS,			// to many ports
	CARD_INFO_READSTATE_AUTOPLAY,			// autoplay value out of range
	CARD_INFO_READSTATE_SCENE_NOT_FOUND,	// requested scene not found
	CARD_INFO_READSTATE_PICO_ERROR			// pico error - probably card read error
};
// pwpstate is 0 if the card is write protected and 1 if the card can be written.

/*
 * susbdmxtwodrv_config_reload() - force to reload the configuration file
 */
BOOL	SUSBDMXTWODRV_TYPE	susbdmxtwodrv_config_reload(IN HANDLE h);

/*
 * susbdmxtwodrv_webserver() - return 1 in state if webserver is available
 */
BOOL	SUSBDMXTWODRV_TYPE	susbdmxtwodrv_webserver(IN HANDLE h, OUT PUCHAR pState);

/*
 * susbdmxtwodrv_factory_defaults - reset Plexus to factory defaults
 * 
 * On Success, the Plexus will disconnect within 10ms and reboot in about 1 second.
 */
BOOL	SUSBDMXTWODRV_TYPE	susbdmxtwodrv_factory_defaults(IN HANDLE);

/*
 * susbdmxtwodrv_format_filesystem
 */
BOOL	SUSBDMXTWODRV_TYPE	susbdmxtwodrv_format_filesystem(IN HANDLE h, OUT PINT pState);

/*
 * susbdmxtwodrv_file_create
 */
BOOL	SUSBDMXTWODRV_TYPE	susbdmxtwodrv_file_create(IN HANDLE h, IN PCSTR pFileName, IN ULONGLONG Size, OUT PINT pState);

/*
 * susbdmxtwodrv_file_remove
 */
BOOL	SUSBDMXTWODRV_TYPE	susbdmxtwodrv_file_remove(IN HANDLE h, IN PCSTR pFileName, OUT PINT pState);

/*
 * susbdmxtwodrv_file_read
 */
BOOL	SUSBDMXTWODRV_TYPE	susbdmxtwodrv_file_read(IN HANDLE h, IN PCSTR pFileName, IN ULONGLONG Offset, IN UINT BufferSize, OUT PUCHAR pBuffer, OUT PULONGLONG FileSize, OUT PINT pState);

/*
 * susbdmxtwodrv_file_write
 */
BOOL	SUSBDMXTWODRV_TYPE	susbdmxtwodrv_file_write(IN HANDLE h, IN PCSTR pFileName, IN ULONGLONG Offset, IN UINT BufferSize, IN PUCHAR pBuffer, OUT PINT pState);

/*
 * susbdmxtwodrv_dir_create
 */
BOOL	SUSBDMXTWODRV_TYPE	susbdmxtwodrv_dir_create(IN HANDLE h, IN PCSTR pFileName, OUT PINT pState);

/*
 * susbdmxtwodrv_dir_remove
 */
BOOL	SUSBDMXTWODRV_TYPE	susbdmxtwodrv_dir_remove(IN HANDLE h, IN PCSTR pFileName, OUT PINT pState);

/*
 * susbdmxtwodrv_dir_read - read directory structure on SD-Card
 */
BOOL	SUSBDMXTWODRV_TYPE	susbdmxtwodrv_dir_read(IN HANDLE h, IN PCSTR pDirectory, IN UINT Offset, OUT PUINT Size, OUT PUINT pTime, OUT PUCHAR pAttribute, OUT PSTR pEntry, IN UINT EntryLenght, OUT PINT pState);

// maximum values for Filename and BufferSize
#define MAX_FILENAME	(256)
#define MAX_BUFFERSIZE	(0xe00)

/*
 *   susbdmxtwodrv_tx(): transmitt a frame using the new protocol on bulk endpoints
 *
 * INPUTs:	h			- handle to the device, returned by usbdmx_open()
 *			universe	- addressed universe
 *			slots		- number of bytes to be transmitted, as well as sizeof(buffer)
 *						  for DMX512: buffer[0] == startcode, slots <= 513
 *			buffer		- data to be transmitted,  !!! sizeof(buffer) >= slots !!!
 *			config		- configuration of the transmitter, see below for possible values
 *			time		- time value in s, depending on config, either timeout or delay
 *			time_break	- break time in s (can be zero, to not transmitt a break)
 *			time_mab	- Mark-after-Break time (can be zero)
 * OUTPUTs:	ptimestamp	- timestamp of this frame in ms, does overrun
 *			pstatus		- status of this transmission, see below for possible values
 */
BOOL	SUSBDMXTWODRV_TYPE	susbdmxtwodrv_tx(	IN HANDLE h, IN UCHAR universe, IN USHORT slots, 
									IN PUCHAR buffer, IN UCHAR config, IN FLOAT time, 
									IN FLOAT time_break, IN FLOAT time_mab, 
									OUT PUSHORT ptimestamp, OUT PUCHAR pstatus);
/*
 * values of config (to be ored together)
 */
#define SUSBDMXTWODRV_BULK_CONFIG_DELAY	(0x01)	// delay frame by time
#define SUSBDMXTWODRV_BULK_CONFIG_BLOCK	(0x02)	// block while frame is not transmitting (timeout given by time)
#define SUSBDMXTWODRV_BULK_CONFIG_RX		(0x04)	// switch to RX after having transmitted this frame
#define SUSBDMXTWODRV_BULK_CONFIG_NORETX	(0x08)	// do not retransmit this frame
#define SUSBDMXTWODRV_BULK_CONFIG_TXIRQ	(0x40)	// send data using transmitter IRQ instead of timer

/*
 *   susbdmxtwodrv_rx(): receive a frame using the new protocol on bulk endpoints
 *
 * INPUTs:	h			- handle to the device, returned by usbdmx_open()
 *			universe	- addressed universe
 *			slots_set	- number of bytes to receive, as well as sizeof(buffer)
 *						  for DMX512: buffer[0] == startcode, slots_set <= 513
 *			buffer		- data to be transmitted,  !!! sizeof(buffer) >= slots !!!
 *			timeout		- timeout for receiving the total frame in s,
 *			timeout_rx	- timeout between two slots used to detect premature end of frames
 * OUTPUTs:	pslots_get	- number of slots actually received, *pslots_get <= slots_set
 *          ptimestamp	- timestamp of this frame in ms, does overrun
 *			pstatus		- status of the reception, see below for possible values
 */
BOOL	SUSBDMXTWODRV_TYPE	susbdmxtwodrv_rx(	IN HANDLE h, IN UCHAR universe, IN USHORT slots_set, 
									IN PUCHAR buffer, IN FLOAT timeout, IN FLOAT timeout_rx,
									OUT PUSHORT pslots_get, OUT PUSHORT ptimestamp, OUT PUCHAR pstatus);

/*
 *   SUSBDMXTWODRV_TXRX(): transmitt and receive a frame using the new protocol on bulk endpoints
 */
BOOL	SUSBDMXTWODRV_TYPE	susbdmxtwodrv_txrx(	IN HANDLE h, IN UCHAR universe, 
										IN USHORT slotstx, IN PUCHAR buffertx,	// data to transmit
										IN USHORT slotsrx, IN PUCHAR bufferrx,	// received data
										IN UCHAR config, IN FLOAT time, 
										IN FLOAT time_break, IN FLOAT time_mab, 
										IN FLOAT timeout_txend_to_rxstart, IN FLOAT timeout_slot_to_slot,
										OUT PUSHORT pslots_get, OUT PUSHORT ptimestamp, OUT PUCHAR pstatus);

/*
 * values of *pstatus
 */
#define SUSBDMXTWODRV_BULK_STATUS_OK				(0x00)
#define SUSBDMXTWODRV_BULK_STATUS_TIMEOUT			(0x01)	// request timed out	
#define SUSBDMXTWODRV_BULK_STATUS_TX_START_FAILED	(0x02)	// delayed start failed
#define SUSBDMXTWODRV_BULK_STATUS_UNIVERSE_WRONG	(0x03)	// wrong universe addressed\tabularnewline
#define SUSBDMXTWODRV_BULK_STATUS_RX_RDM_LENGTH	(0x08)	// rdm length decoded
#define SUSBDMXTWODRV_BULK_STATUS_RX_OLD_FRAME	(0x10)	// old frame not read
#define SUSBDMXTWODRV_BULK_STATUS_RX_TIMEOUT		(0x20)	// receiver finished with timeout (ored with others)
#define SUSBDMXTWODRV_BULK_STATUS_RX_NO_BREAK		(0x40)	// frame without break received (ored with others)
#define SUSBDMXTWODRV_BULK_STATUS_RX_FRAMEERROR	(0x80)	// frame finished with frame error (ored with others)

/*
 * macro to check, it the return status is ok
 */
#define SUSBDMXTWODRV_BULK_STATUS_IS_OK(s) (s == SUSBDMXTWODRV_BULK_STATUS_OK)

/******************************************************************************
 * High-Level functions facilitating the above
 */

/*
 * SUSBDMXTWODRV_DMXTX - send dmx data
 *
 * INPUTs:	h			- handle to the device, returned by usbdmx_open()
 *			universe	- addressed universe
 *			slots		- number of bytes to transmitt without startcode, as well as sizeof(buffer)
 *			buffer		- data to be transmitted without startcode,  !!! sizeof(buffer) >= slots !!!
 * RETURN:	== 0		- data transmitted
 *			!= 0		- data not transmitted
 */
int SUSBDMXTWODRV_TYPE susbdmxtwodrv_dmxtx(HANDLE h, UCHAR universe, USHORT slots, const void *buffer);

/*
 * SUSBDMXTWODRV_DMXRX - receive dmx data with startcode 0
 *
 * INPUTs:	h			- handle to the device, returned by usbdmx_open()
 *			universe	- addressed universe
 *			buffersize	- size of buffer
 *			buffer		- received data without startcode
 * RETURN:	== 0		- no data received, or data with non-DMX startcode
 *			>  0		- amount of received data
 *			<  0		- reception error
 */
int SUSBDMXTWODRV_TYPE susbdmxtwodrv_dmxrx(HANDLE h, UCHAR universe, USHORT buffersize, void *buffer);
// return values for dmxrx and tx
enum {
	SUSBDMXTWODRV_DMX_INPUT		= -1,	// input parameter wrong
	SUSBDMXTWODRV_DMX_TX			= -10,	// transmission failed
	SUSBDMXTWODRV_DMX_UNIVERSE	= -11,	// universe number wrong
	SUSBDMXTWODRV_DMX_RX			= -20,	// reception failed
	SUSBDMXTWODRV_DMX_RX_NOBREAK	= -23,	// reception failed with no break
};


/*
 * SUSBDMXTWODRV_RDM - exchange rdm data
 *
 * INPUTs:	h				- handle to the device, returned by usbdmx_open()
 *			universe		- addressed universe
 *			txlen			- number of bytes to transmitt
 *			txbuffer		- data to be transmitted,  !!! sizeof(buffer) >= txlen !!!
 *							  START code, Sub START Code and Message Length are inserted befor,
 *							  Checksum is appended behind the data.
 *          rxsize          - size of receive buffer (set to 0 if not data is to be received)
 *          rxbuffer        - receive buffer
 *                            Checksum is validated and stipped
 *                            START code, Sub START code and length are validated and stipped
 * RETURN:	> 0				- length of received data without START codes, length and Checksum
 *                            (can be larger than rxsize if rxsize was to small)
 *			< 0				- handling/reception error
 */
int SUSBDMXTWODRV_TYPE susbdmxtwodrv_rdm(HANDLE h, UCHAR universe, UCHAR txlen, const void *txbuffer, UCHAR rxsize, void *rxbuffer);

/*
 * SUSBDMXTWODRV_RDMDISCOVERY - exchange rdm discovery data (handle special response without break)
 *
 * INPUTs:	h				- handle to the device, returned by usbdmx_open()
 *			universe		- addressed universe
 *			txlen			- number of bytes to transmitt
 *			txbuffer		- data to be transmitted,  !!! sizeof(buffer) >= txlen !!!
 *							  START code, Sub START Code and Message Length are inserted befor,
 *							  Checksum is appended behind the data.
 *          rxsize          - size of receive buffer
 *          rxbuffer        - receive buffer, contains received UIDs
 * RETURN:	> 0				- length of UIDs (can be larger than rxsize if rxsize was to small to fit all uids)
 *			< 0				- handling/reception error
 */
int SUSBDMXTWODRV_TYPE susbdmxtwodrv_rdmdiscovery(HANDLE h, UCHAR universe, UCHAR txlen, const void *txbuffer, UCHAR rxsize, void *rxbuffer);

// return values for rdm() and rdmdiscovery():
enum {
	SUSBDMXTWODRV_RDM_INPUT			= -1,	// input parameter wrong
	SUSBDMXTWODRV_RDM_TX				= -10,	// transmission failed
	SUSBDMXTWODRV_RDM_UNIVERSE		= -11,	// universe number wrong
	SUSBDMXTWODRV_RDM_RX				= -20,	// reception failed
	SUSBDMXTWODRV_RDM_RX_TIMEOUT		= -21,	// reception failed with timed out
	SUSBDMXTWODRV_RDM_RX_FRAMEERROR	= -22,	// reception failed with frame error
	SUSBDMXTWODRV_RDM_RX_NOBREAK		= -23,	// reception failed with no break
	SUSBDMXTWODRV_RDM_RX_LENGTH		= -24,	// reception failed with length error (received less data than expected)
	SUSBDMXTWODRV_RDM_RX_STARTCODE	= -25,	// wrong startcode received
	SUSBDMXTWODRV_RDM_RX_SUBSTARTCODE	= -26,	// wrong sub-startcode received
	SUSBDMXTWODRV_RDM_RX_CHECKSUM		= -27,	// reception failed with checksum error
	SUSBDMXTWODRV_RDM_COLLISION		= -30	// collision detected
};

/*
 * SUSBDMXTWODRV_HIGHLIGHT - highlight the device
 */
BOOL	SUSBDMXTWODRV_TYPE susbdmxtwodrv_highlight(HANDLE h, BOOL state);

#if defined __cplusplus
}
#endif

#endif // SUSBDMXTWODRV_H
